# RESTful API Sample
* This sample shows some calls to ActiveReports Server RESTful API from modern Angular application
* See API calls [description](http://arhelp.grapecity.com/webhelp/AR12Server/WebAPI/index.html)
* It is simple Angular based web application, so look at next files to understand integration
	* `app/ars-rest.service.ts` — implementation of some ActiveReports Server RESTful API calls
	* `app/pdf-print.service.ts` — implementation of browser—specific printing

## How to run

### Prerequisites
* Install latest [Node 8.x.x](https://nodejs.org/dist/latest-v8.x/)
* Install [yarn](https://yarnpkg.com/lang/en/docs/install/)
* ActiveReports Server should be configured to allow CORS requests: please add the following lines in the `system.webServer` section of web.config file located in `ActiveReports Server Installation folder\site` directory:
```xml
<httpProtocol>
        <customHeaders>
                <add name="Access-Control-Allow-Origin" value="*" />
                <add name="Access-Control-Allow-Headers" value="AuthToken, Authorization, Origin, Content-Type, Accept, X-Requested-With" />
                <add name="Access-Control-Expose-Headers" value="X-ARS-DocumentId" />
        </customHeaders>
</httpProtocol>
```
### Run using the command line
* Go to project directory
* Run `yarn install` to install all dependencies
* Run `yarn start` to start server
* Open [localhost:54512](http://localhost:54512/) in browser
* Note: you can use `yarn build` command to build the application

## Implementation details
* The sample is implemented with Angular. Here you can look into a few related resources:
	* Angular:
		* [Official documentation](https://angular.io/docs/ts/latest/)
		* [Very useful blog](https://yakovfain.com/)
	* Material Design for Angular:
		* [material.angular.io](https://material.angular.io/)
		* [Material Icons Guide](http://google.github.io/material-design-icons/)
	* Internationalization:
		* [Angular's internationalization](https://angular.io/docs/ts/latest/cookbook/i18n.html)
		* [ngx-translate](https://github.com/ngx-translate/core)
	* TypeScript:
		* [Official site](https://www.typescriptlang.org/)
		* [Style guide](https://angular.io/styleguide)
		* [Angular: Why TypeScript?](https://vsavkin.com/writing-angular-2-in-typescript-1fa77c78d8e8#.8mpo89bso)
* Some files description
	* `package.json`, `tsconfig.json`, `.angular-cli.json` — Angular [configuration](https://angular.io/docs/ts/latest/quickstart.html#!#create-and-configure) files
	* `web.config` — IIS/IIS Express configurations
	* `src/index.html`, `src/app/app.component.ts` — main page and component with selector tag [app-root](https://angular.io/docs/ts/latest/quickstart.html#!#index)
	* `src/main.ts` — the application [entry point](https://angular.io/docs/ts/latest/quickstart.html#!#main)
	* `src/app/app.routing.ts` — [navigation settings](https://angular.io/docs/ts/latest/guide/router.html)
	* `src/app/auth.service.ts`, `src/app/login/login.component.ts` — authentication implementation (see [here](http://4dev.tech/2016/03/login-screen-and-authentication-with-angular2/) and [here](https://www.angular-meteor.com/tutorials/socially/angular2/angular-material-and-custom-angular-auth-forms))
	* `src/app/preview/preview.component.ts` — private view, which required authentication
	* `src/app/ars-rest.service.ts` — RESTful API calls to ActiveReports Server
