import { BrowserModule } from '@angular/platform-browser';
import { NgModule } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { APP_BASE_HREF } from '@angular/common';
import { HttpModule, Http } from '@angular/http'
import { MaterialModule, MdButtonModule, MdCheckboxModule } from '@angular/material';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';

import { AppComponent } from './app.component';
import { LoginComponent } from './login/login.component';
import { PreviewComponent } from './preview/preview.component';
import { PdfViewerComponent } from './pdf-viewer/pdf-viewer.component';
import { PrintServerComponent } from './print-server/print-server.component';
import { routing } from './app-routing';
import { ArsRestService } from './ars-rest.service';
import { AuthService } from './auth.service';
import { PdfPrintService } from './pdf-print.service';
import { DesignerComponent } from './designer/designer.component';

export function createTranslateLoader(http: Http) {
	return new TranslateHttpLoader(http, './assets/i18n/', '.json');
}

@NgModule({
	imports: [
		BrowserModule,
		// i18n
		TranslateModule.forRoot({
			loader: {
				provide: TranslateLoader,
				useFactory: (createTranslateLoader),
				deps: [Http]
			}
		}),
		// "Material Design for Angular"
		MaterialModule, MdButtonModule, MdCheckboxModule,
		BrowserAnimationsModule,
		// Angular modules
		BrowserModule,
		FormsModule,
		// AJAX module
		HttpModule,
		// Routing module
		routing
	],
	entryComponents: [
		PrintServerComponent // required for dialog: http://stackoverflow.com/a/40185852
	],
	declarations: [
		AppComponent,
		LoginComponent,
		PreviewComponent,
		PrintServerComponent,
		PdfViewerComponent,
		DesignerComponent
	],
	providers: [
		// specify base path: https://angular.io/docs/ts/latest/api/common/index/APP_BASE_HREF-let.html
		{ provide: APP_BASE_HREF, useValue: '/' },

		ArsRestService,
		AuthService,
		PdfPrintService
	],
	bootstrap: [AppComponent]
})
export class AppModule { }
