import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';

import { ArsRestService } from './ars-rest.service';

/**
 * Login/logout service with navigation to login screen.
 */
@Injectable()
export class AuthService {

	private _errorMsgString: Subject<string>;

	constructor(
		private _router: Router,
		private _restService: ArsRestService) {
		this._errorMsgString = new Subject<string>();
	}

	/**
	 * Login implementation.
	 */
	login(server: string, username: string, password: string): Promise<void> {
		return this._restService.login(server, username, password).then(
			() => {
				this._router.navigate(['/']);
			},
			(err: any) => {
				this._errorMsgString.next(err);
			});
	}

	/**
	 * Performs logout.
	 */
	logout(): void {
		this._restService.logout();
		this._errorMsgString.next('');
		this._router.navigate(['/login']);
	}

	/**
	 * Checks credentials and request if necessary.
	 */
	checkCredentials(): boolean {
		if (!this._restService.hasCredentials()) {
			this._errorMsgString.next('');
			this._router.navigate(['/login']);
			return false;
		}
		return true;
	}

	/**
	 * Gets error message.
	 */
	get errorMsg(): Observable<string> {
		return this._errorMsgString.asObservable();
	}
}
