import { Injectable } from '@angular/core';

/**
 * Client-side printing implementation through PDF.
 */
@Injectable()
export class PdfPrintService {

	private supportPdfFrame: boolean;

	constructor() {
		var isEdge = navigator.userAgent.toLowerCase().indexOf('edge') > -1;
		var isFirefox = navigator.userAgent.toLowerCase().indexOf('firefox') > -1;
		var isChrome = navigator.userAgent.toLowerCase().indexOf('chrome') > -1;
		var isSafari = navigator.userAgent.toLowerCase().indexOf('safari') > -1;
		var isOpera = navigator.userAgent.toLowerCase().indexOf('opera') > -1;

		this.supportPdfFrame = isChrome || isSafari || isOpera;
	}


	/**
	 * Open print dialog as a iframe.
	 * @param url - path to PDF
	 */
	useFrame(url: string): void {
		var printingFrameId = 'print-frame';
		var iframe = document.getElementById('#' + printingFrameId);
		if (!iframe) {
			iframe = document.createElement('iframe');
			iframe.setAttribute('id', printingFrameId);
			document.body.appendChild(iframe);
		}
		iframe.setAttribute('src', encodeURI(url));
	}


	/**
	 * Open print dialog in a new window.
	 * @param url - path to PDF
	 */
	openInWindow(url: string): boolean {
		var printWindow: Window;
		try {
			printWindow = window.open(url, '_blank');
			if (printWindow) printWindow.focus();
		} catch (e) { }
		return !!printWindow;
	}

	/**
	 * Open print dialog.
	 * @param url - path to PDF
	 */
	print(url: string): void {
		if (this.supportPdfFrame) {
			this.useFrame(url);
			return;
		}

		if (!this.openInWindow(url)) {
			window.location.href = url;
		}
	}
}
